
% The Wholesale Electricty Marlet of Russia
% day ahead prices for the European price zone
% time steps are given in Moscow Timezone

clear; clc;

%% Initialisation

% Upload price archive in matlab format

load priceDataEurPriceZone priceDataEurPriceZone;

% Forecast problem details

ForecastMomentT = datetime('03.09.2012 23:00', 'InputFormat','dd.MM.yyyy HH:mm');     % Forecast moment
P = 24;                                                                 % Forecast horizon

% Forecast model parameters

M = 144;                % Extrapolation model on the most similar pattern parameter
loopStep = 24;          % Additional variable for looping


%% Algorithm (according to description http://www.mbureau.energy)

% 1. Define the latest available actual pattern.

T = find(priceDataEurPriceZone.timestep == ForecastMomentT);

if ~isempty(T)
    
    patternLatestAvailable.timestep = priceDataEurPriceZone.timestep(T-M+1:T);
    patternLatestAvailable.value = priceDataEurPriceZone.value(T-M+1:T);
    
    checkValues = patternLatestAvailable.value == 0;
    if sum(double(checkValues)) == M
        fprintf('Error: patternLatestAvailable contains only zeros, check the data!');
        return
    end
        
else
    
    fprintf('Error: forecast moment %s is absent in the time series, please check!', datestr(T));
    return
    
end

% 2. Find the most similar pattern.

k = T - M - P;

similarityMeasure.timeDelay = [];
similarityMeasure.value = [];

% Looping through timeseries history
while k > M
    
    patternTemp.value = priceDataEurPriceZone.value(k+1 : k+M);
    patternTemp.timestep = priceDataEurPriceZone.timestep(k+1 : k+M);
    similarityMeasure.timeDelay(end+1) = T - k;
    
    checkValues = patternTemp.value == 0;
    if sum(double(checkValues)) == M                   
        similarityMeasure.value(end+1) = 0;
    else
        similarityMeasure.value(end+1) = abs(corr(patternTemp.value, patternLatestAvailable.value, 'type', 'Pearson'));
    end
    
    k = k - loopStep;       % looping not through hours, but through days
    
end

% Pick up the most similar pattern

maximumSimilarity = max(similarityMeasure.value);
msp = similarityMeasure.timeDelay(similarityMeasure.value == maximumSimilarity);
fprintf('maximumSimilarity = %1.3f with timeDelay = %d\n', maximumSimilarity, msp);

patternMostSimilar.timestep = priceDataEurPriceZone.timestep( T-msp+1 : T-msp+M );
patternMostSimilar.value = priceDataEurPriceZone.value( T-msp+1 : T-msp+M );

% abs(corr(patternMostSimilar.value, patternLatestAvailable.value, 'type', 'Pearson'))

% 3. Calculate the linear coefficients.

A = regress(patternLatestAvailable.value, [patternMostSimilar.value ones(size(patternMostSimilar.value))]);

% 4. Define the base pattern.

patternBase.timestep = priceDataEurPriceZone.timestep( T-msp+M+1 : T-msp+M+P );
patternBase.value = priceDataEurPriceZone.value( T-msp+M+1 : T-msp+M+P );

% 5. Calculate forecast values.

patternForecast.timestep = priceDataEurPriceZone.timestep( T+1 : T+P );
patternForecast.value = [patternBase.value ones(size(patternBase.value))] * A;

% Estimate an error

patternActuals.timestep = priceDataEurPriceZone.timestep( T+1 : T+P );
patternActuals.value = priceDataEurPriceZone.value( T+1 : T+P );

MAE = mean(abs(patternForecast.value - patternActuals.value));
MAPE = mean(abs(patternForecast.value - patternActuals.value) ./ patternActuals.value);

%% Plot results and error

fprintf('Foreast error: MAE = %3.2f Rub/MWh, MAPE = %2.2f%%\n', MAE, MAPE*100);

plot(patternActuals.timestep, [patternActuals.value patternForecast.value])
title(sprintf('Day ahead price forecast for Electricity Market of Russia, forecast moment %s, MAE = %3.2f Rub/MWh, MAPE = %2.2f%%',...
    datestr(T), MAE, MAPE*100))
